@@ -1,2 +1,2 @@
 from mdxpy.mdx import Member, CalculatedMember, MdxBuilder, MdxTuple, MdxHierarchySet, normalize, MdxSet, ElementType, \
-    Order, MdxLevelExpression
+    Order, MdxPropertiesTuple, DimensionProperty, MdxLevelExpression
