@@ -130,6 +130,39 @@ class Member:
         return hash(self.unique_name)
 
 
+class DimensionProperty(Member):
+    SHORT_NOTATION = False
+
+    def __init__(self, dimension: str, hierarchy: str, attribute: str):
+        super(DimensionProperty, self).__init__(dimension, hierarchy, attribute)
+
+    @staticmethod
+    def from_unique_name(unique_name: str) -> 'DimensionProperty':
+        dimension = Member.dimension_name_from_unique_name(unique_name)
+        attribute = Member.element_name_from_unique_name(unique_name)
+        if unique_name.count("].[") == 1:
+            return DimensionProperty(dimension, dimension, attribute)
+
+        elif unique_name.count("].[") == 2:
+            hierarchy = Member.hierarchy_name_from_unique_name(unique_name)
+            return DimensionProperty(dimension, hierarchy, attribute)
+
+        else:
+            raise ValueError(f"Argument '{unique_name}' must be a valid DimensionProperty unique name")
+
+    @staticmethod
+    def of(*args: str) -> 'DimensionProperty':
+        # case: '[dim].[elem]'
+        if len(args) == 1:
+            return DimensionProperty.from_unique_name(args[0])
+        elif len(args) == 2:
+            return DimensionProperty(args[0], args[0], args[1])
+        elif len(args) == 3:
+            return DimensionProperty(*args)
+        else:
+            raise ValueError("method takes either one, two or three str arguments")
+
+
 class CalculatedMember(Member):
     def __init__(self, dimension: str, hierarchy: str, element: str, calculation: str):
         super(CalculatedMember, self).__init__(dimension, hierarchy, element)
@@ -248,6 +281,39 @@ class MdxTuple:
         return len(self.members)
 
 
+class MdxPropertiesTuple:
+
+    def __init__(self, members):
+        self.members = list(members)
+
+    @staticmethod
+    def of(*args: Union[str, DimensionProperty]) -> 'MdxPropertiesTuple':
+        # handle unique element names
+        members = [DimensionProperty.of(member)
+                   if isinstance(member, str) else member
+                   for member in args]
+        mdx_tuple = MdxPropertiesTuple(members)
+        return mdx_tuple
+
+    @staticmethod
+    def empty() -> 'MdxPropertiesTuple':
+        return MdxPropertiesTuple.of()
+
+    def add_member(self, member: Union[str, DimensionProperty]):
+        if isinstance(member, str):
+            member = DimensionProperty.of(member)
+        self.members.append(member)
+
+    def is_empty(self) -> bool:
+        return not self.members
+
+    def to_mdx(self) -> str:
+        return f"{','.join(member.unique_name for member in self.members)}"
+
+    def __len__(self):
+        return len(self.members)
+
+
 class MdxSet:
 
     @abstractmethod
@@ -420,6 +486,10 @@ class MdxHierarchySet(MdxSet):
             end_member = Member.of(end_member)
         return RangeHierarchySet(start_member, end_member)
 
+    @staticmethod
+    def unions(sets: List['MdxHierarchySet'], allow_duplicates: bool = False) -> 'MdxHierarchySet':
+        return MultiUnionHierarchySet(sets, allow_duplicates)
+
     def filter_by_attribute(self, attribute_name: str, attribute_values: List,
                             operator: Optional[str] = '=') -> 'MdxHierarchySet':
         return FilterByAttributeHierarchySet(self, attribute_name, attribute_values, operator)
@@ -443,6 +513,9 @@ class MdxHierarchySet(MdxSet):
     def tm1_sort(self, ascending=True) -> 'MdxHierarchySet':
         return Tm1SortHierarchySet(self, ascending)
 
+    def hierarchize(self) -> 'MdxHierarchySet':
+        return HierarchizeSet(self)
+
     def head(self, head: int) -> 'MdxHierarchySet':
         return HeadHierarchySet(self, head)
 
@@ -824,6 +897,17 @@ class Tm1SortHierarchySet(MdxHierarchySet):
         return f"{{TM1SORT({self.underlying_hierarchy_set.to_mdx()},{'ASC' if self.ascending else 'DESC'})}}"
 
 
+class HierarchizeSet(MdxHierarchySet):
+
+    def __init__(self, underlying_hierarchy_set: MdxHierarchySet):
+        super(HierarchizeSet, self).__init__(underlying_hierarchy_set.dimension,
+                                                underlying_hierarchy_set.hierarchy)
+        self.underlying_hierarchy_set = underlying_hierarchy_set
+
+    def to_mdx(self) -> str:
+        return f"{{HIERARCHIZE({self.underlying_hierarchy_set.to_mdx()})}}"
+
+
 class HeadHierarchySet(MdxHierarchySet):
 
     def __init__(self, underlying_hierarchy_set: MdxHierarchySet, head: int):
@@ -940,6 +1024,26 @@ class GenerateAttributeToMemberSet(MdxHierarchySet):
                f"{{STRTOMEMBER('[{self.dimension}].[{self.hierarchy}].[' + [{self.underlying_hierarchy_set.dimension}].[{self.underlying_hierarchy_set.hierarchy}].CURRENTMEMBER.PROPERTIES(\"{self.attribute}\") + ']')}})}}"
 
 
+class MultiUnionHierarchySet(MdxHierarchySet):
+
+    def __init__(self, sets: List[MdxHierarchySet], allow_duplicates: bool = False):
+        if not sets:
+            raise RuntimeError('sets must not be empty')
+
+        super(MultiUnionHierarchySet, self).__init__(
+            sets[0].dimension,
+            sets[0].hierarchy)
+
+        self.sets = sets
+        self.allow_duplicates = allow_duplicates
+
+    def to_mdx(self) -> str:
+        if self.allow_duplicates:
+            return f"{{{','.join(set_.to_mdx() for set_ in self.sets)}}}"
+        else:
+            return f"{{{' + '.join(set_.to_mdx() for set_ in self.sets)}}}"
+
+
 class MdxAxis:
     def __init__(self):
         self.tuples: List[MdxTuple] = list()
@@ -989,6 +1093,8 @@ class MdxBuilder:
         self.cube = normalize(cube)
         self.axes = {0: MdxAxis.empty()}
         self._where = MdxTuple.empty()
+        # dimension properties by axis
+        self.axes_properties = {0: MdxPropertiesTuple.empty()}
         self.calculated_members = list()
         self._tm1_ignore_bad_tuples = False
 
@@ -1069,6 +1175,13 @@ class MdxBuilder:
         self._where.add_member(member)
         return self
 
+    def add_member_to_properties(self, axis: int, member: Union[str, DimensionProperty]) -> 'MdxBuilder':
+        if axis in self.axes_properties:
+            self.axes_properties[axis].add_member(member)
+        else:
+            self.axes_properties[axis] = MdxPropertiesTuple([member])
+        return self
+
     def where(self, *args: Union[str, Member]) -> 'MdxBuilder':
         for member in args:
             if isinstance(member, str):
@@ -1078,6 +1191,37 @@ class MdxBuilder:
             self.add_member_to_where(member)
         return self
 
+    def add_properties_to_row_axis(self, *args: Union[str, DimensionProperty]) -> 'MdxBuilder':
+        return self.add_properties(1, *args)
+
+    def add_properties_to_column_axis(self, *args: Union[str, DimensionProperty]) -> 'MdxBuilder':
+        return self.add_properties(0, *args)
+
+    def add_properties(self, axis: int, *args: Union[str, DimensionProperty]) -> 'MdxBuilder':
+        for member in args:
+            if isinstance(member, str):
+                member = DimensionProperty.of(member)
+
+            if not isinstance(member, DimensionProperty):
+                raise ValueError(f"Argument '{member}' must be of type str or DimensionProperty")
+
+            self.add_member_to_properties(axis, member)
+
+        return self
+
+    def _axis_mdx(self, position):
+        axis = self.axes[position]
+        axis_properties = self.axes_properties.get(position, MdxPropertiesTuple.empty())
+        if axis.is_empty():
+            return ""
+
+        return " ".join([
+            axis.to_mdx(self._tm1_ignore_bad_tuples),
+            "DIMENSION PROPERTIES",
+            "MEMBER_NAME" if axis_properties.is_empty() else axis_properties.to_mdx(),
+            f"ON {position}"
+        ])
+
     def to_mdx(self) -> str:
         mdx_with = "WITH\r\n" + "\r\n".join(
             calculated_member.to_mdx()
@@ -1085,9 +1229,9 @@ class MdxBuilder:
             in self.calculated_members) + "\r\n"
 
         mdx_axes = ",\r\n".join(
-            f"{'' if axis.is_empty() else (axis.to_mdx(self._tm1_ignore_bad_tuples) + ' ON ' + str(position))}"
-            for position, axis
-            in self.axes.items())
+            self._axis_mdx(position)
+            for position
+            in self.axes)
 
         mdx_where = "\r\nWHERE " + self._where.to_mdx() if not self._where.is_empty() else ""
 
