@@ -2,7 +2,8 @@ import unittest
 
 import pytest
 
-from mdxpy import Member, MdxTuple, MdxHierarchySet, normalize, MdxBuilder, CalculatedMember, MdxSet, Order, ElementType, MdxLevelExpression
+from mdxpy import DimensionProperty, Member, MdxTuple, MdxHierarchySet, normalize, MdxBuilder, CalculatedMember, MdxSet, \
+    Order, ElementType, MdxLevelExpression
 
 
 class Test(unittest.TestCase):
@@ -224,7 +225,7 @@ class Test(unittest.TestCase):
 
         self.assertEqual(
             "{ ([dimension1].[dimension1].[element1],[dimension2].[dimension2].[element3]),"
-            "([dimension1].[dimension1].[element2],[dimension2].[dimension2].[element2])," 
+            "([dimension1].[dimension1].[element2],[dimension2].[dimension2].[element2]),"
             "([dimension1].[dimension1].[element3],[dimension2].[dimension2].[element1]) }",
             mdx_set.to_mdx())
 
@@ -426,6 +427,13 @@ class Test(unittest.TestCase):
             "{TM1SORT({TM1SUBSETALL([dimension].[dimension])},DESC)}",
             hierarchy_set.to_mdx())
 
+    def test_mdx_hierarchy_set_hierarchize(self):
+        hierarchy_set = MdxHierarchySet.tm1_subset_all("Dimension").hierarchize()
+
+        self.assertEqual(
+            "{HIERARCHIZE({TM1SUBSETALL([dimension].[dimension])})}",
+            hierarchy_set.to_mdx())
+
     def test_mdx_hierarchy_descendants(self):
         member = Member.of('Dimension', 'Hierarchy', 'Member1')
         hierarchy_set = MdxHierarchySet.descendants(member)
@@ -452,8 +460,8 @@ class Test(unittest.TestCase):
         member = Member.of('Dimension', 'Hierarchy', 'Member1')
         hierarchy_set = MdxHierarchySet.descendants(member,
                                                     MdxLevelExpression.level_number(2,
-                                                                                  'Dimension',
-                                                                                  'Hierarchy'),
+                                                                                    'Dimension',
+                                                                                    'Hierarchy'),
                                                     desc_flag='SELF_AND_BEFORE')
         self.assertEqual("{DESCENDANTS([dimension].[hierarchy].[member1], "
                          "[dimension].[hierarchy].LEVELS(2), "
@@ -473,7 +481,9 @@ class Test(unittest.TestCase):
     def test_mdx_hierarchy_set_range(self):
         member1 = Member.of('Dimension', 'Hierarchy', 'Member1')
         member2 = Member.of('Dimension', 'Hierarchy', 'Member2')
+
         hierarchy_set = MdxHierarchySet.range(member1, member2)
+
         self.assertEqual("{[dimension].[hierarchy].[member1]:[dimension].[hierarchy].[member2]}",
                          hierarchy_set.to_mdx())
 
@@ -607,6 +617,19 @@ class Test(unittest.TestCase):
             "{STRTOMEMBER('[manager].[manager].[' + [store].[store].CURRENTMEMBER.PROPERTIES(\"Manager\") + ']')})}",
             hierarchy_set.to_mdx())
 
+    def test_mdx_hierarchy_set_unions_allow_duplicates(self):
+        hierarchy_set = MdxSet.unions([
+            MdxHierarchySet.children(Member.of("Dimension", "element1")),
+            MdxHierarchySet.member(Member.of("Dimension", "element2")),
+            MdxHierarchySet.member(Member.of("Dimension", "element3"))
+        ], True)
+
+        self.assertEqual(
+            "{{[dimension].[dimension].[element1].CHILDREN},"
+            "{[dimension].[dimension].[element2]},"
+            "{[dimension].[dimension].[element3]}}",
+            hierarchy_set.to_mdx())
+
     def test_mdx_builder_simple(self):
         mdx = MdxBuilder.from_cube("cube") \
             .rows_non_empty() \
@@ -618,8 +641,28 @@ class Test(unittest.TestCase):
 
         self.assertEqual(
             "SELECT\r\n"
-            "NON EMPTY {[dim2].[dim2].[elem2]} ON 0,\r\n"
-            "NON EMPTY {TM1FILTERBYLEVEL({TM1SUBSETALL([dim1].[dim1])},0)} ON 1\r\n"
+            "NON EMPTY {[dim2].[dim2].[elem2]} DIMENSION PROPERTIES MEMBER_NAME ON 0,\r\n"
+            "NON EMPTY {TM1FILTERBYLEVEL({TM1SUBSETALL([dim1].[dim1])},0)} DIMENSION PROPERTIES MEMBER_NAME ON 1\r\n"
+            "FROM [cube]\r\n"
+            "WHERE ([dim3].[dim3].[elem3],[dim4].[dim4].[elem4])",
+            mdx)
+
+    def test_mdx_builder_simple_properties(self):
+        mdx = MdxBuilder.from_cube("cube") \
+            .rows_non_empty() \
+            .add_hierarchy_set_to_row_axis(MdxHierarchySet.all_leaves("Dim1")) \
+            .add_properties_to_row_axis(DimensionProperty.of("Dim1", "Code and Name")) \
+            .columns_non_empty() \
+            .add_hierarchy_set_to_column_axis(MdxHierarchySet.member(Member.of("Dim2", "Elem2"))) \
+            .add_properties_to_column_axis(DimensionProperty.of("Dim2", "Name")) \
+            .where(Member.of("Dim3", "Elem3"), Member.of("Dim4", "Elem4")) \
+            .to_mdx()
+
+        self.assertEqual(
+            "SELECT\r\n"
+            "NON EMPTY {[dim2].[dim2].[elem2]} DIMENSION PROPERTIES [dim2].[dim2].[name] ON 0,\r\n"
+            "NON EMPTY {TM1FILTERBYLEVEL({TM1SUBSETALL([dim1].[dim1])},0)} "
+            "DIMENSION PROPERTIES [dim1].[dim1].[codeandname] ON 1\r\n"
             "FROM [cube]\r\n"
             "WHERE ([dim3].[dim3].[elem3],[dim4].[dim4].[elem4])",
             mdx)
@@ -636,8 +679,8 @@ class Test(unittest.TestCase):
 
         self.assertEqual(
             "SELECT\r\n"
-            "NON EMPTY TM1IGNORE_BADTUPLES {[dim2].[dim2].[elem2]} ON 0,\r\n"
-            "NON EMPTY TM1IGNORE_BADTUPLES {TM1FILTERBYLEVEL({TM1SUBSETALL([dim1].[dim1])},0)} ON 1\r\n"
+            "NON EMPTY TM1IGNORE_BADTUPLES {[dim2].[dim2].[elem2]} DIMENSION PROPERTIES MEMBER_NAME ON 0,\r\n"
+            "NON EMPTY TM1IGNORE_BADTUPLES {TM1FILTERBYLEVEL({TM1SUBSETALL([dim1].[dim1])},0)} DIMENSION PROPERTIES MEMBER_NAME ON 1\r\n"
             "FROM [cube]\r\n"
             "WHERE ([dim3].[dim3].[elem3],[dim4].[dim4].[elem4])",
             mdx)
@@ -649,7 +692,7 @@ class Test(unittest.TestCase):
 
         self.assertEqual(
             "SELECT\r\n"
-            "{[dim1].[dim1].[elem1]} ON 0\r\n"
+            "{[dim1].[dim1].[elem1]} DIMENSION PROPERTIES MEMBER_NAME ON 0\r\n"
             "FROM [cube]",
             mdx)
 
@@ -664,11 +707,11 @@ class Test(unittest.TestCase):
 
         self.assertEqual(
             "SELECT\r\n"
-            "{[dim1].[dim1].[elem1]} ON 0,\r\n"
-            "{[dim2].[dim2].[elem2]} ON 1,\r\n"
-            "{[dim3].[dim3].[elem3]} ON 2,\r\n"
-            "{[dim4].[dim4].[elem4]} ON 3,\r\n"
-            "{[dim5].[dim5].[elem5]} ON 4\r\n"
+            "{[dim1].[dim1].[elem1]} DIMENSION PROPERTIES MEMBER_NAME ON 0,\r\n"
+            "{[dim2].[dim2].[elem2]} DIMENSION PROPERTIES MEMBER_NAME ON 1,\r\n"
+            "{[dim3].[dim3].[elem3]} DIMENSION PROPERTIES MEMBER_NAME ON 2,\r\n"
+            "{[dim4].[dim4].[elem4]} DIMENSION PROPERTIES MEMBER_NAME ON 3,\r\n"
+            "{[dim5].[dim5].[elem5]} DIMENSION PROPERTIES MEMBER_NAME ON 4\r\n"
             "FROM [cube]",
             mdx)
 
@@ -682,8 +725,8 @@ class Test(unittest.TestCase):
 
         self.assertEqual(
             "SELECT\r\n"
-            "NON EMPTY {[dim2].[dim2].[elem2]} ON 0,\r\n"
-            "NON EMPTY {TM1FILTERBYLEVEL({TM1SUBSETALL([dim1].[dim1])},0)} ON 1\r\n"
+            "NON EMPTY {[dim2].[dim2].[elem2]} DIMENSION PROPERTIES MEMBER_NAME ON 0,\r\n"
+            "NON EMPTY {TM1FILTERBYLEVEL({TM1SUBSETALL([dim1].[dim1])},0)} DIMENSION PROPERTIES MEMBER_NAME ON 1\r\n"
             "FROM [cube]",
             mdx)
 
@@ -725,8 +768,37 @@ class Test(unittest.TestCase):
             "MEMBER [period].[period].[avg2016] AS AVG({[period].[period].[2016].CHILDREN},"
             "[cube].([dim1].[dim1].[totaldim1],[dim2].[dim2].[totaldim2]))\r\n"
             "SELECT\r\n"
-            "NON EMPTY {([period].[period].[avg2016])} ON 0,\r\n"
-            "NON EMPTY {TM1FILTERBYLEVEL({TM1SUBSETALL([dim1].[dim1])},0)} ON 1\r\n"
+            "NON EMPTY {([period].[period].[avg2016])} DIMENSION PROPERTIES MEMBER_NAME ON 0,\r\n"
+            "NON EMPTY {TM1FILTERBYLEVEL({TM1SUBSETALL([dim1].[dim1])},0)} DIMENSION PROPERTIES MEMBER_NAME ON 1\r\n"
+            "FROM [cube]\r\n"
+            "WHERE ([dim2].[dim2].[totaldim2])",
+            mdx)
+
+    def test_mdx_builder_with_calculated_member_with_properties(self):
+        mdx = MdxBuilder.from_cube(cube="Cube").with_member(
+            CalculatedMember.avg(
+                dimension="Period",
+                hierarchy="Period",
+                element="AVG 2016",
+                cube="Cube",
+                mdx_set=MdxHierarchySet.children(member=Member.of("Period", "2016")),
+                mdx_tuple=MdxTuple.of(Member.of("Dim1", "Total Dim1"),
+                                      Member.of("Dim2", "Total Dim2")))) \
+            .rows_non_empty() \
+            .add_hierarchy_set_to_row_axis(MdxHierarchySet.all_leaves("dim1", "dim1")) \
+            .columns_non_empty() \
+            .add_member_tuple_to_columns(Member.of("Period", "AVG 2016")) \
+            .where("[Dim2].[Total Dim2]") \
+            .add_properties_to_row_axis("[Dim1].[Code and Name]") \
+            .to_mdx()
+
+        self.assertEqual(
+            "WITH\r\n"
+            "MEMBER [period].[period].[avg2016] AS AVG({[period].[period].[2016].CHILDREN},"
+            "[cube].([dim1].[dim1].[totaldim1],[dim2].[dim2].[totaldim2]))\r\n"
+            "SELECT\r\n"
+            "NON EMPTY {([period].[period].[avg2016])} DIMENSION PROPERTIES MEMBER_NAME ON 0,\r\n"
+            "NON EMPTY {TM1FILTERBYLEVEL({TM1SUBSETALL([dim1].[dim1])},0)} DIMENSION PROPERTIES [dim1].[dim1].[codeandname] ON 1\r\n"
             "FROM [cube]\r\n"
             "WHERE ([dim2].[dim2].[totaldim2])",
             mdx)
@@ -765,8 +837,8 @@ class Test(unittest.TestCase):
             "MEMBER [period].[period].[sum2016] AS SUM({[period].[period].[2016].CHILDREN},"
             "[cube].([dim1].[dim1].[totaldim1],[dim2].[dim2].[totaldim2]))\r\n"
             "SELECT\r\n"
-            "NON EMPTY {[period].[period].[avg2016],[period].[period].[sum2016]} ON 0,\r\n"
-            "NON EMPTY {TM1FILTERBYLEVEL({TM1SUBSETALL([dim1].[dim1])},0)} ON 1\r\n"
+            "NON EMPTY {[period].[period].[avg2016],[period].[period].[sum2016]} DIMENSION PROPERTIES MEMBER_NAME ON 0,\r\n"
+            "NON EMPTY {TM1FILTERBYLEVEL({TM1SUBSETALL([dim1].[dim1])},0)} DIMENSION PROPERTIES MEMBER_NAME ON 1\r\n"
             "FROM [cube]\r\n"
             "WHERE ([dim2].[dim2].[totaldim2])",
             mdx)
@@ -843,8 +915,8 @@ class Test(unittest.TestCase):
         self.assertEqual(
             mdx,
             "SELECT\r\n"
-            "{TM1SUBSETALL([dimension].[dimension])} ON 0,\r\n"
-            "{} ON 1\r\n"
+            "{TM1SUBSETALL([dimension].[dimension])} DIMENSION PROPERTIES MEMBER_NAME ON 0,\r\n"
+            "{} DIMENSION PROPERTIES MEMBER_NAME ON 1\r\n"
             "FROM [cube]")
 
     def test_add_empty_set_to_axis_error(self):
@@ -874,14 +946,11 @@ class Test(unittest.TestCase):
     def test_level_expression_number(self):
         level = MdxLevelExpression.level_number(8, "Dimension1", "Hierarchy1")
         self.assertEqual("[dimension1].[hierarchy1].LEVELS(8)", level.to_mdx())
-        print(level.to_mdx())
 
     def test_level_expression_name(self):
         level = MdxLevelExpression.level_name('NamedLevel', "Dimension1", "Hierarchy1")
         self.assertEqual("[dimension1].[hierarchy1].LEVELS('NamedLevel')", level.to_mdx())
-        print(level.to_mdx())
 
     def test_level_expression_member_level(self):
         level = MdxLevelExpression.member_level(Member.of("Dimension1", "Hierarchy1", "Element1"))
-        self.assertEqual("[dimension1].[hierarchy1].[element1].LEVEL", level.to_mdx())
-        print(level.to_mdx())
+        self.assertEqual("[dimension1].[hierarchy1].[element1].LEVEL", level.to_mdx())
\ No newline at end of file
